function [theta_est, theta_err, LLF, ctable_mod, ctable_fit, psyc_mate, d0, sivals] = mADC_model_fit_psyc_del_2(ctable, cvals, InputStruct)

%% function that fits the m-ADC model to data in a contingency table
% rather than fitting d's to individual stimulus strengths, fits the entire
% psychophysical function in one shot
%
%  Sridhar Devarajan, Sep 2016
%  sridhar@cns.iisc.ernet.in
%
% Usage:
%   [theta_est, theta_err, LLF, ctable_fit] = mADC_model_fit_psyc(ctable, cvals)
%
% ctable is a 3D contingency table of the form (m+1) x (m+1) x k,
% cvals is a 1D array of stimulus strength of size 1xk; *must* be normalized
% from 0-1, where 1 is the expected stimulus strength for asymptotic dmax
%
% where:
%   m is the number of stimulus/response locations + 1 catch/NoGo
%   k is the number of stimulus strengths tested (e.g. contrast)
%
%   data are expected to be arranged in increasing order of stimulus strength
%   the last row of each ctable should be identical and contain the false
%   alarm counts across all locations


% model type: 'u' for unequal sensitivities and 'e' for equal sensitivities
global model_type
model_type = 'e';

if size(ctable, 1) ~= size(ctable, 2)
    error('number of rows and columns of contingency table have to match');
end

if size(ctable, 3) ~= length(cvals)
    error('third dimension of ctable should be same length as cvals');
end

global M K
M = size(ctable, 1) - 1;
K = size(ctable, 3);

global yobs
for i = 1:K
    yobs{i} = squeeze(ctable(:,:,i));
    %     yobs{i}(end,:) = (1/K) * yobs{i}(end,:);        % we divide the false alarm counts (last row of each table) by the number of stimulus strengths -- this prevents "overfitting" to the false alarm rates
end

global svals
svals = cvals;

%%% start the fitting

%% initialize

dmax_init = 2.0*ones(1,M); dmax_lb = 1e-4;  dmax_ub = 5.0;
dmax_Lim = [dmax_lb*ones(1,M); dmax_ub*ones(1,M)];

n_init = 2.0*ones(1,M); n_lb = 2.0; n_ub = 2.0;
n_Lim = [n_lb*ones(1,M); n_ub*ones(1,M)];

s50_init = 0.5*max(svals)*ones(1,M); s50_lb = 0; s50_ub = max(svals);
s50_Lim = [s50_lb*ones(1,M); s50_ub*ones(1,M)];

c_init = rand(1,M); c_lb = -5; c_ub = 5;
c_Lim = [c_lb*ones(1,M); c_ub*ones(1,M)];

del_init = zeros(1,M); del_lb = -10; del_ub = 10;
del_Lim = [del_lb*ones(1,M); del_ub*ones(1,M)];

Aeq = []; Beq = [];
Aueq = []; Bueq = [];

global d0flag;
d0flag = 0;

parm_names = {'dmax_init', 'dmax_Lim', 'n_init', 'n_Lim', 's50_init', 's50_Lim', 'c_init', 'c_Lim', 'del_init', 'del_Lim', 'Aueq', 'Bueq', 'Aeq', 'Beq', 'd0flag'};

for parm = 1:length(parm_names)
    if isfield(InputStruct, parm_names{parm})
        if ~isempty(eval(['InputStruct.' parm_names{parm}]))
            eval([parm_names{parm} '=' 'InputStruct.' parm_names{parm}]);
        end
    end        
end

% disp(d0flag);

theta_init = [dmax_init, n_init, s50_init, c_init, del_init];


%% constraints
fprintf([sprintf('dmax_lb = %0.4f',dmax_lb) '\n']);

lb = [dmax_Lim(1,:), n_Lim(1,:), s50_Lim(1,:), c_Lim(1,:), del_Lim(1,:)];
ub = [dmax_Lim(2,:), n_Lim(2,:), s50_Lim(2,:), c_Lim(2,:), del_Lim(2,:)];

%%% additional  constraints here
%Aeq = [zeros(1,m*k) zeros(1,k)]; Beq = [0.0];

%% fitting
[mLe_est, mLe_val, exitflag, output, lambda, grad, hessian] = fmincon(@madc_LLF, theta_init, Aueq, Bueq, Aeq, Beq, lb, ub);

theta_est = mLe_est;
LLF       = -mLe_val; % -ve, because func value minimized, so LLF is maximized

theta_se   = sqrt(inv(hessian));
theta_err = diag(theta_se, 0);

%% the best estimate and parameters
py = get_madc_probs(theta_est);

ctable_fit = cell(1,K);
for i=1:K
    ctable_fit{i} = py{i} .* repmat(sum(yobs{i}, 2), 1, M+1);
end

ctable_mod = yobs;

%% let's try some plotting - Ahh Nope, not here!!

dmaxe   = theta_est(1:M);
ne      = theta_est(M+1:2*M);
s50e    = theta_est(2*M+1:3*M);
ce      = theta_est(3*M+1:4*M);
del     = theta_est(4*M+1:5*M);

[psyc_mate, d0, sivals] = get_interp_psycmat(dmaxe, ne, s50e, del);


%%
function F = madc_LLF (theta)
%% evaluate the LLF based on fit of real data to cont table produced by these values of d/c

global yobs K

py = get_madc_probs(theta);

LLF = 0;
for i = 1:K
    LLF = LLF + sum(sum(yobs{i} .* log(py{i})));
end

F   = -LLF;

%%
function py = get_madc_probs(theta)
% return a k element cell array with an m+1 x m+1 contingency table for
% each stimulus strength

global model_type M K

dmax = theta(1:M); n = theta(M+1:2*M); s50 = theta(2*M+1:3*M); del = theta(4*M+1:5*M);

[psyc_mat, d0] = get_psycmat(dmax, n, s50, del);

py = cell(1,K);
for i = 1:K
    d = psyc_mat(:, i)';  % get the d' value from the psychophysical function matrix for that stimulus strength (i) for all locations 1:M
    c = theta(3*M+1:4*M);  % theta(end-M+1:end);
    
    dMat = repmat(d0',[M+1 1]);
    for loc = 1:M
        dMat(loc,loc) = d(loc);
    end
    
    py{i} = madc_pcont_table_del(dMat, c, model_type);
end

%%
function [dmat, d0] = get_psycmat(dmax, n, s50, del)
% create an matrix of psychophysical functions for each location
global M K svals d0flag

dmat = nan(M, K);
for i = 1:M
    d0(i,1) = (rectF(del(i)).^n(i))./(rectF(del(i)).^n(i) + s50(i)^n(i));
    dmat(i,:) = dmax(i) * rectF(((rectF(svals + del(i)).^n(i))./(rectF(svals + del(i)).^n(i) + s50(i).^n(i))) - d0flag*d0(i,1));
end


%%
function [dmat, d0, sivals] = get_interp_psycmat(dmax, n, s50, del)
% create an matrix of psychophysical functions for each location
global M  svals d0flag

sivals = linspace(min(svals), max(svals), 1000);
dmat = nan(M, length(sivals));
for i = 1:M
    d0(i,1) = (rectF(del(i)).^n(i))./(rectF(del(i)).^n(i) + s50(i)^n(i));
    dmat(i,:) = dmax(i) * rectF(((rectF(sivals + del(i)).^n(i))./(rectF(sivals + del(i)).^n(i) + s50(i).^n(i))) - d0flag*d0(i,1));
end


%%











